// glfw and glew
#include <iostream>
# include <GL/glew.h>
# include <GLFW/glfw3.h>
# include <math.h>

//SHADER START_________________________________________________________
const GLchar * vertex_shader_source =
"\
   # version 150 core\n\
   in vec2 position;\
   in vec3 color;\
   out vec3 Color;\
   void main() { Color = color; gl_Position = vec4(position, 0, 1); }\
";

const GLchar * fragment_shader_source =
"\
    # version 150 core\n\
    in vec3 Color;\
    out vec4 outColor;\
    void main() { outColor = vec4(Color, 1.0); }\
";
//SHADER END __________________________________________________________
// Callbacks-----------------------
   void key_stokes  ( GLFWwindow* , int , int ,int ,int);
   void mouse_button( GLFWwindow* , int, int, int );
   void mouse_cursor( GLFWwindow* , double, double );
   void mouse_cursor_entered( GLFWwindow* window, int entered);
//----------------------------------
//--------------------------------------
#define GRIDX 100
struct Vertex{ GLfloat x,y,r,g,b;};
struct Vertex axisy_vertex[]=
{
    0.0,-0.7, 0, 1, 0, //vertex format x,y,z,r,g,b
    0.0, 0.7, 0, 1, 0
};
struct Vertex axisx_vertex[]=
{
    -0.7, 0.0, 1, 0, 0,
     0.7, 0.0, 1, 0, 0
};
struct Vertex vertex[GRIDX];

void init_vertices(void)
{
 int i;
        for ( i=0;i<GRIDX;i++)
        {
	vertex[i].x = -1.0 +i*2.0/(GRIDX-1);
        vertex[i].y = 0.0;
        vertex[i].r = 1.0;
        vertex[i].g = 1.0;
        vertex[i].b = 1.0;
        }
}
//--------------------------------------
	int mouse_window_entered;
//--------------------------------------
int main (int argc, char * argv[])
{
   GLFWwindow* window;
   int i;
 

    glfwInit();
    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 2);
    glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
    glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
    glfwWindowHint(GLFW_RESIZABLE, GL_FALSE);

    window = glfwCreateWindow(500, 300, "open-gl glfw + glew", NULL, NULL);
	if (window==NULL)
        { std::cout << "failed to create window"<<std::endl;
          glfwTerminate();
          return (-1);
        }

    glfwMakeContextCurrent(window);
//===================================
// glew Begin
    glewExperimental = GL_TRUE;
    glewInit();
// glew END
//===================================

        // ---- MAKE SHADERS ---- //

// Shader Begin---------------------------
    GLuint vertex_shader = glCreateShader(GL_VERTEX_SHADER);
    glShaderSource(vertex_shader, 1, &vertex_shader_source, NULL);
    glCompileShader(vertex_shader);

    GLuint fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
    glShaderSource(fragment_shader, 1, &fragment_shader_source, NULL);
    glCompileShader(fragment_shader);

    GLuint shader_program = glCreateProgram();
    glAttachShader(shader_program, vertex_shader);
    glAttachShader(shader_program, fragment_shader);
    glBindFragDataLocation(shader_program, 0, "outColor");
    glLinkProgram(shader_program);

    glUseProgram(shader_program);
//Shader End-------------------------------

//buffer object
    GLuint vbo;
    glGenBuffers(1, &vbo);
    glBindBuffer(GL_ARRAY_BUFFER, vbo);
//vertex object
    GLuint vao;
    glGenVertexArrays(1, &vao);
    glBindVertexArray(vao);

    GLint posAttrib = glGetAttribLocation(shader_program, "position");
    GLint colAttrib = glGetAttribLocation(shader_program, "color");

    glEnableVertexAttribArray(posAttrib);
    glEnableVertexAttribArray(colAttrib);

    glVertexAttribPointer(posAttrib, 2, GL_FLOAT, GL_FALSE, 5*sizeof(float), 0);
    glVertexAttribPointer(colAttrib, 3, GL_FLOAT, GL_FALSE, 5*sizeof(float), (void*)(2*sizeof(float)));
//-------------------------

//== init vertex array
       init_vertices();
//== set callbacks
        glfwSetKeyCallback(window,key_stokes);
        glfwSetMouseButtonCallback(window, mouse_button);
        glfwSetCursorPosCallback(window, mouse_cursor);
        glfwSetCursorEnterCallback(window, mouse_cursor_entered);
//==
    while (!glfwWindowShouldClose(window))
    {
        glClearColor(0,0,0,1);
        glClear(GL_COLOR_BUFFER_BIT);


//generate data points
	for(i=0;i<GRIDX;i++){
	  if(vertex[i].x>-0.7&&vertex[i].x<0.7)
	  vertex[i].y = sin( i*2*3.14159/(GRIDX-1));
	};
//plot data
	glBufferData(GL_ARRAY_BUFFER,sizeof(vertex),vertex, GL_DYNAMIC_DRAW);
	glDrawArrays(GL_LINE_STRIP, 0, GRIDX);
//plot y-axis
	glBufferData(GL_ARRAY_BUFFER,sizeof(axisy_vertex),axisy_vertex, GL_STATIC_DRAW);
	glDrawArrays(GL_LINE_STRIP, 0, 2);
//plot x-axis
	glBufferData(GL_ARRAY_BUFFER,sizeof(axisx_vertex),axisx_vertex, GL_STATIC_DRAW);
	glDrawArrays(GL_LINE_STRIP, 0, 2);

        glfwSwapBuffers(window);
        glfwPollEvents();
    }

        // ---- CLEAN UP ---- //

    glDeleteProgram(shader_program);
    glDeleteShader(vertex_shader);
    glDeleteShader(fragment_shader);
    glDeleteVertexArrays(1, &vao);
    glDeleteBuffers(1, &vbo);

    glfwTerminate();

    return 0;
}
//++++++++++++++++++++++++++++++++=
// key strokes
//++++++++++++++++++++++++++++++++=
void key_stokes( GLFWwindow* window, int key , int scancode ,int action,int mods)
{       
        if(action !=GLFW_PRESS)
        return;
        switch(key)
                {
                case GLFW_KEY_ESCAPE: 
                        glfwSetWindowShouldClose(window,GLFW_TRUE);
                        break;
                case GLFW_KEY_0:
                        break;
                case GLFW_KEY_C:
                        break;
                default:
                        break;
                }
}
//++++++++++++++++++++++++++++++++=
// mouse_cursor
// 0,0
//       WIDTH,HEIGHT
//++++++++++++++++++++++++++++++++=
void mouse_cursor( GLFWwindow* window, double x, double y)
{
	extern int mouse_window_entered;
	if(mouse_window_entered==1)std::cout<<x<<","<<y<<std::endl;
}
//++++++++++++++++++++++++++++++++=
// mouse_cursor_entered
//++++++++++++++++++++++++++++++++=
void mouse_cursor_entered( GLFWwindow* window, int entered)
{
	extern int mouse_window_entered;
	if(entered)
	{
	mouse_window_entered=1;
	std::cout<< "entered window"<<std::endl;
	}
	else
	{
	mouse_window_entered=0;
	std::cout<< "out of  window"<<std::endl;
	}
}
//++++++++++++++++++++++++++++++++=
// mouse_button
//++++++++++++++++++++++++++++++++=
void mouse_button( GLFWwindow* window, int key , int action, int mods)
{
        if(action !=GLFW_PRESS)
        return;

        switch(key)
                {
                case GLFW_MOUSE_BUTTON_LEFT:
        		std::cout << "pushed left  mouse button"<<std::endl;
                        glfwSetWindowShouldClose(window,GLFW_TRUE);
                        break;
                case GLFW_MOUSE_BUTTON_RIGHT:
        		std::cout << "pushed right mouse button"<<std::endl;
                       break;
                case GLFW_MOUSE_BUTTON_MIDDLE:
        		std::cout << "pushed middle mouse button"<<std::endl;
                        break;
                default:
                        break;
                }

}

